@@ -4,7 +4,7 @@ use 5.006002;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 require XSLoader;
 XSLoader::load('Crypt::NSS', $VERSION);
@@ -30,7 +30,7 @@ sub import {
     return if $initialized;
     
     my $config_dir;
-    my $cipher_suite = "Export";
+    my $cipher_suite = "US";
     
     while(@_) {
         my $arg = shift;
